from typing import Any, Optional

def kml(
    request: Any, label: Any, model: Any, field_name: Optional[Any] = ..., compress: bool = ..., using: Any = ...
): ...
def kmz(request: Any, label: Any, model: Any, field_name: Optional[Any] = ..., using: Any = ...): ...
