from typing import Any

from django.contrib.gis.geos.base import GEOSBase as GEOSBase

class GEOSCoordSeq(GEOSBase):
    ptr_type: Any = ...
    def __init__(self, ptr: Any, z: bool = ...) -> None: ...
    def __iter__(self) -> Any: ...
    def __len__(self): ...
    def __getitem__(self, index: Any): ...
    def __setitem__(self, index: Any, value: Any) -> None: ...
    def getOrdinate(self, dimension: Any, index: Any): ...
    def setOrdinate(self, dimension: Any, index: Any, value: Any) -> None: ...
    def getX(self, index: Any): ...
    def setX(self, index: Any, value: Any) -> None: ...
    def getY(self, index: Any): ...
    def setY(self, index: Any, value: Any) -> None: ...
    def getZ(self, index: Any): ...
    def setZ(self, index: Any, value: Any) -> None: ...
    @property
    def size(self): ...
    @property
    def dims(self): ...
    @property
    def hasz(self): ...
    def clone(self): ...
    @property
    def kml(self): ...
    @property
    def tuple(self): ...
    @property
    def is_counterclockwise(self): ...
