from typing import Any

class WKTAdapter:
    wkt: Any = ...
    srid: Any = ...
    def __init__(self, geom: Any) -> None: ...
    def __eq__(self, other: Any) -> Any: ...
    def __hash__(self) -> Any: ...
