from typing import Any, Optional

class SpatialOperator:
    sql_template: Any = ...
    op: Any = ...
    func: Any = ...
    def __init__(self, op: Optional[Any] = ..., func: Optional[Any] = ...) -> None: ...
    @property
    def default_template(self): ...
    def as_sql(self, connection: Any, lookup: Any, template_params: Any, sql_params: Any): ...
