from typing import Any, Optional

from django.contrib.sitemaps import Sitemap as Sitemap

class KMLSitemap(Sitemap):
    geo_format: str = ...
    locations: Any = ...
    def __init__(self, locations: Optional[Any] = ...) -> None: ...
    def items(self): ...
    def location(self, obj: Any): ...

class KMZSitemap(KMLSitemap):
    geo_format: str = ...
