from typing import Any, Optional

from django.db.models import Func as Func

class IntervalToSeconds(Func):
    function: str = ...
    template: str = ...
    def __init__(self, expression: Any, *, output_field: Optional[Any] = ..., **extra: Any) -> None: ...

class SecondsToInterval(Func):
    function: str = ...
    template: str = ...
    def __init__(self, expression: Any, *, output_field: Optional[Any] = ..., **extra: Any) -> None: ...
