from typing import Any

class SpatialRefSysMixin:
    @property
    def srs(self): ...
    @property
    def ellipsoid(self): ...
    @property
    def name(self): ...
    @property
    def spheroid(self): ...
    @property
    def datum(self): ...
    @property
    def projected(self): ...
    @property
    def local(self): ...
    @property
    def geographic(self): ...
    @property
    def linear_name(self): ...
    @property
    def linear_units(self): ...
    @property
    def angular_name(self): ...
    @property
    def angular_units(self): ...
    @property
    def units(self): ...
    @classmethod
    def get_units(cls, wkt: Any): ...
    @classmethod
    def get_spheroid(cls, wkt: Any, string: bool = ...): ...
