from typing import Any, Optional

def add_srs_entry(
    srs: Any,
    auth_name: str = ...,
    auth_srid: Optional[Any] = ...,
    ref_sys_name: Optional[Any] = ...,
    database: Optional[Any] = ...,
) -> None: ...
