from django.contrib.gis.db.backends.base.features import BaseSpatialFeatures as BaseSpatialFeatures
from django.db.backends.mysql.features import DatabaseFeatures as MySQLDatabaseFeatures

class DatabaseFeatures(BaseSpatialFeatures, MySQLDatabaseFeatures):
    has_spatialrefsys_table: bool = ...
    supports_add_srs_entry: bool = ...
    supports_distance_geodetic: bool = ...
    supports_length_geodetic: bool = ...
    supports_area_geodetic: bool = ...
    supports_transform: bool = ...
    supports_null_geometries: bool = ...
    supports_num_points_poly: bool = ...
    def supports_empty_geometry_collection(self): ...
    def supports_geometry_field_unique_index(self): ...
