from typing import Any

from .base import GEOSBase as GEOSBase

class PreparedGeometry(GEOSBase):
    ptr_type: Any = ...
    destructor: Any = ...
    ptr: Any = ...
    def __init__(self, geom: Any) -> None: ...
    def contains(self, other: Any): ...
    def contains_properly(self, other: Any): ...
    def covers(self, other: Any): ...
    def intersects(self, other: Any): ...
    def crosses(self, other: Any): ...
    def disjoint(self, other: Any): ...
    def overlaps(self, other: Any): ...
    def touches(self, other: Any): ...
    def within(self, other: Any): ...
