from typing import Any

from django.db import models as models

class AreaField(models.FloatField):
    geo_field: Any = ...
    def __init__(self, geo_field: Any) -> None: ...
    def get_prep_value(self, value: Any): ...
    def get_db_prep_value(self, value: Any, connection: Any, prepared: bool = ...): ...
    def from_db_value(self, value: Any, expression: Any, connection: Any): ...
    def get_internal_type(self): ...

class DistanceField(models.FloatField):
    geo_field: Any = ...
    def __init__(self, geo_field: Any) -> None: ...
    def get_prep_value(self, value: Any): ...
    def get_db_prep_value(self, value: Any, connection: Any, prepared: bool = ...): ...
    def from_db_value(self, value: Any, expression: Any, connection: Any): ...
    def get_internal_type(self): ...
